import 'dart:io';

import 'package:_fe_analyzer_shared/src/scanner/token.dart';
import 'package:dart_style/dart_style.dart';
import 'package:analyzer/source/line_info.dart';
import 'package:report/parser/logger.dart';

void printToken(Token token) {
  while (!token.isEof) {
    logInfo('${token.lexeme} ');
    if (token.precedingComments != null) {
      logInfo(' ${token.precedingComments?.lexeme ?? ""} \n');
      Token nextPrecedingComment = token.precedingComments?.next;
      while (nextPrecedingComment != null) {
        print(' ${nextPrecedingComment?.lexeme ?? ""}  \n');
        nextPrecedingComment = nextPrecedingComment.next;
      }
    }
    token = token.next;
  }
}

void printTokenRange(
  Token fromToken,
  StringBuffer sb,
  LineInfo lineInfo,
) {
  int endOfLast = fromToken.end;
  Token token = fromToken;
  int lineNumber = lineInfo.getLocation(token.offset).lineNumber;
  while (!token.isEof) {
    if (token.precedingComments != null) {
      sb.write(' \n ${token.precedingComments?.lexeme ?? ""}  \n');
      Token nextPrecedingComment = token.precedingComments?.next;
      while (nextPrecedingComment != null) {
        sb.write('  ${nextPrecedingComment?.lexeme ?? ""}  \n');
        nextPrecedingComment = nextPrecedingComment.next;
      }
    }
    if (token.offset > endOfLast) {
      sb.write(" ");
    }
    int currentNumber = lineInfo.getLocation(token.offset).lineNumber;
    int diff = currentNumber - lineNumber;
    if (diff > 1 && token.precedingComments == null) {
      for (int i = 0; i < diff; i++) {
        sb.write('\n');
      }
    }
    // else if (diff > 2 && token.precedingComments != null) {
    //   for (int i = 0; i < diff; i++) {
    //     sb.write('\n');
    //   }
    // }
    lineNumber = currentNumber;
    // print('token ${token.offset} ${token} ${token.runtimeType} line ${lineInfo.getLocation(token.offset).lineNumber}');

    sb.write(token.lexeme);
    endOfLast = token.end;

    token = token.next;
  }
}

void writeToken(Token token, String path, LineInfo lineInfo) {
  StringBuffer stringBuffer = StringBuffer();
  printTokenRange(token, stringBuffer, lineInfo);
  // print('== \n ${stringBuffer.toString()} \n');

  DartFormatter dartFormatter = DartFormatter(lineEnding: "\n", pageWidth: 160);
  String formatterStr = dartFormatter.format(stringBuffer.toString());
  File file = File(path);
  file.writeAsString(formatterStr);
}
